From 7aa5bef4a0949846ec565e42b9799e6c910aa492 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Mattias=20Engdeg=C3=A5rd?= <mattiase@acm.org>
Date: Tue, 29 Dec 2020 16:55:06 +0100
Subject: [PATCH] Pretty-print keys without <> around modifiers (bug#45536)

Be consistent when pretty-printing keys: put modifiers outside <>,
thus the more logical C-M-<return> instead of <C-M-return>.
This behaviour is controlled by modifiers-inside-angle-brackets,
by default t (for compatibility).

* src/keymap.c (Fsingle_key_description): Optionally skip modifier
prefix before adding <>.
(syms_of_keymap): Add modifiers-inside-angle-brackets.
---
 src/keymap.c | 34 +++++++++++++++++++++++++++++-----
 1 file changed, 29 insertions(+), 5 deletions(-)

diff --git a/src/keymap.c b/src/keymap.c
index ca2d33dba4..c167772311 100644
--- a/src/keymap.c
+++ b/src/keymap.c
@@ -2013,6 +2013,9 @@ DEFUN ("key-description", Fkey_description, Skey_description, 1, 2, 0,
 Optional arg PREFIX is the sequence of keys leading up to KEYS.
 For example, [?\C-x ?l] is converted into the string \"C-x l\".
 
+Modifiers are put inside angle brackets iff `modifiers-inside-angle-brackets'
+is non-nil.
+
 For an approximate inverse of this, see `kbd'.  */)
   (Lisp_Object keys, Lisp_Object prefix)
 {
@@ -2229,6 +2232,8 @@ DEFUN ("single-key-description", Fsingle_key_description,
 Control characters turn into C-whatever, etc.
 Optional argument NO-ANGLES non-nil means don't put angle brackets
 around function keys and event symbols.
+Modifiers are put inside angle brackets iff `modifiers-inside-angle-brackets'
+is non-nil.
 
 See `text-char-description' for describing character codes.  */)
   (Lisp_Object key, Lisp_Object no_angles)
@@ -2260,11 +2265,23 @@ DEFUN ("single-key-description", Fsingle_key_description,
     {
       if (NILP (no_angles))
 	{
-	  Lisp_Object result;
-	  char *buffer = SAFE_ALLOCA (sizeof "<>"
-				      + SBYTES (SYMBOL_NAME (key)));
-	  esprintf (buffer, "<%s>", SDATA (SYMBOL_NAME (key)));
-	  result = build_string (buffer);
+	  Lisp_Object namestr = SYMBOL_NAME (key);
+	  const char *sym = SSDATA (namestr);
+	  ptrdiff_t len = SBYTES (namestr);
+	  /* Optionally find the extent of the modifier prefix, like "C-M-",
+	     to avoid putting modifiers around them. */
+	  int i = 0;
+	  if (!modifiers_inside_angle_brackets)
+	    while (i < len - 3 && sym[i + 1] == '-'
+		   && strchr ("CMSsHA", sym[i]))
+	      i += 2;
+	  char *buffer = SAFE_ALLOCA (len + 3);
+	  memcpy (buffer, sym, i);
+	  buffer[i] = '<';
+	  memcpy (buffer + i + 1, sym + i, len - i);
+	  buffer [len + 1] = '>';
+	  buffer [len + 2] = '\0';
+	  Lisp_Object result = build_string (buffer);
 	  SAFE_FREE ();
 	  return result;
 	}
@@ -3331,6 +3348,13 @@ syms_of_keymap (void)
   Vwhere_is_preferred_modifier = Qnil;
   where_is_preferred_modifier = 0;
 
+  DEFVAR_BOOL ("modifiers-inside-angle-brackets",
+	       modifiers_inside_angle_brackets,
+	       doc: /* Whether modifiers go inside <> in key descriptions.
+When non-nil, <return> with a Control modifier is written <C-return>;
+when nil, it is written C-<return>. */);
+  modifiers_inside_angle_brackets = true;
+
   DEFSYM (Qmenu_bar, "menu-bar");
   DEFSYM (Qmode_line, "mode-line");
 
-- 
2.21.1 (Apple Git-122.3)

